@@ -23,12 +23,16 @@
 #include <gtk/gtk.h>
 #include <panel-applet.h>
 #include "vinagre-bookmarks.h"
-#include "vinagre-mdns.h"
+#include "vinagre-bookmarks-entry.h"
 #include "vinagre-utils.h"
 #include "vinagre-connection.h"
 #include "vinagre-commands.h"
 #include <config.h>
 
+#ifdef VINAGRE_ENABLE_AVAHI
+#include "vinagre-mdns.h"
+#endif
+
 static void
 menu_position (GtkMenu    *menu,
 	       gint       *x,
@@ -93,18 +97,61 @@ open_connection_cb (GtkMenuItem *item,
 			    NULL,
 			    &err))
     {
-      gchar *tmp;
-
-      tmp = g_strdup_printf (_("Could not run vinagre: %s"), err->message);
-      vinagre_utils_show_error (tmp, NULL);
-
+      vinagre_utils_show_error (_("Could not run vinagre:"), err->message, NULL);
       g_error_free (err);
-      g_free (tmp);
     }
 
   g_strfreev (argv);
 }
 
+static void
+fill_recursive_menu (GSList *entries, GtkWidget *menu)
+{
+  GSList    *l;
+  GtkWidget *item, *image, *child;
+
+  for (l = entries; l; l = l->next)
+    {
+      VinagreBookmarksEntry *entry = VINAGRE_BOOKMARKS_ENTRY (l->data);
+      VinagreConnection     *conn;
+
+      switch (vinagre_bookmarks_entry_get_node (entry))
+	{
+	  case VINAGRE_BOOKMARKS_ENTRY_NODE_FOLDER:
+	    image = gtk_image_new_from_icon_name ("folder", GTK_ICON_SIZE_MENU);
+	    item = gtk_image_menu_item_new_with_label (vinagre_bookmarks_entry_get_name (entry));
+	    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+					   image);
+	    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+				   item);
+
+	    child = gtk_menu_new ();
+	    gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), child);
+
+	    fill_recursive_menu (vinagre_bookmarks_entry_get_children (entry),
+				 child);
+	    break;
+
+	  case VINAGRE_BOOKMARKS_ENTRY_NODE_CONN:
+	    conn = vinagre_bookmarks_entry_get_conn (entry);
+	    image = gtk_image_new_from_icon_name ("application-x-vnc", GTK_ICON_SIZE_MENU);
+	    item = gtk_image_menu_item_new_with_label (vinagre_connection_get_name (conn));
+	    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+					   image);
+
+	    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+				   item);
+	    g_object_set_data (G_OBJECT (item), "conn", conn);
+	    g_signal_connect (item, "activate", G_CALLBACK (open_connection_cb), NULL);
+	    break;
+
+	  default:
+	    g_assert_not_reached ();
+	}
+    }
+
+}
+
 static void
 fill_menu (GSList *all, GtkWidget *menu)
 {
@@ -117,18 +164,7 @@ fill_menu (GSList *all, GtkWidget *menu)
   item = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-  for (; all; all=all->next)
-    {
-      image = gtk_image_new_from_icon_name ("application-x-vnc", GTK_ICON_SIZE_MENU);
-      item = gtk_image_menu_item_new_with_label (vinagre_connection_get_name (all->data));
-      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-				     image);
-
-      gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-			     item);
-      g_object_set_data (G_OBJECT (item), "conn", all->data);
-      g_signal_connect (item, "activate", G_CALLBACK (open_connection_cb), NULL);
-    }
+  fill_recursive_menu (all, menu);
 }
 
 static void
@@ -139,13 +175,8 @@ open_vinagre_cb (GtkMenuItem *item,
 
   if (!g_spawn_command_line_async ("vinagre", &err))
     {
-      gchar *tmp;
-
-      tmp = g_strdup_printf (_("Could not run vinagre: %s"), err->message);
-      vinagre_utils_show_error (tmp, NULL);
-
+      vinagre_utils_show_error (_("Could not run vinagre:"), err->message, NULL);
       g_error_free (err);
-      g_free (tmp);
     }
 }
 
@@ -173,8 +204,10 @@ click_cb (GtkWidget      *applet,
   all = vinagre_bookmarks_get_all (vinagre_bookmarks_get_default ());
   fill_menu (all, menu);
 
+#ifdef VINAGRE_ENABLE_AVAHI
   all = vinagre_mdns_get_all (vinagre_mdns_get_default ());
   fill_menu (all, menu);
+#endif
 
   gtk_widget_show_all (menu);
   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, (GtkMenuPositionFunc) menu_position, applet, 
@@ -205,13 +238,16 @@ vinagre_applet_fill (PanelApplet *applet,
 {
   GtkWidget *image, *button;
   gchar *tmp;
-  VinagreMdns *mdns;
   static const BonoboUIVerb menu_verbs[] = {
     BONOBO_UI_VERB ("VinagreHelp", help_cb),
     BONOBO_UI_VERB ("VinagreAbout", about_cb),
     BONOBO_UI_VERB_END
   };
 
+#ifdef VINAGRE_ENABLE_AVAHI
+  VinagreMdns *mdns;
+#endif
+
   if (strcmp (iid, "OAFIID:GNOME_VinagreApplet") != 0)
     return FALSE;
 
@@ -238,7 +274,9 @@ vinagre_applet_fill (PanelApplet *applet,
   gtk_container_add (GTK_CONTAINER (applet), image);
   gtk_widget_show_all (GTK_WIDGET (applet));
 
+#ifdef VINAGRE_ENABLE_AVAHI
   mdns = vinagre_mdns_get_default ();
+#endif
 
   return TRUE;
 }
