@@ -22,9 +22,10 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
 #include <glib/gi18n.h>
-#include <glib/goption.h>
 #include <gtk/gtk.h>
+#include <locale.h>
 
 #include "vinagre-connection.h"
 #include "vinagre-commands.h"
@@ -33,10 +34,13 @@
 #include "vinagre-app.h"
 #include "vinagre-utils.h"
 #include "vinagre-prefs.h"
-#include "vinagre-mdns.h"
 #include "vinagre-bacon.h"
 #include <vncdisplay.h>
 
+#ifdef VINAGRE_ENABLE_AVAHI
+#include "vinagre-mdns.h"
+#endif
+
 /* command line */
 static gchar **files = NULL;
 static gchar **remaining_args = NULL;
@@ -70,13 +74,13 @@ vinagre_main_process_command_line (VinagreWindow *window)
     {
       for (i = 0; files[i]; i++) 
 	{
-	  conn = vinagre_connection_new_from_file (files[i], &error);
+	  conn = vinagre_connection_new_from_file (files[i], &error, FALSE);
 	  if (conn)
 	    servers = g_slist_prepend (servers, conn);
 	  else
 	    {
 	      errors = g_slist_prepend (errors,
-					g_strdup_printf ("%s: %s",
+					g_strdup_printf ("<i>%s</i>: %s",
 							files[i],
 							error ? error : _("Unknown error")));
 	      if (error)
@@ -90,12 +94,12 @@ vinagre_main_process_command_line (VinagreWindow *window)
     {
       for (i = 0; remaining_args[i]; i++) 
 	{
-	  conn = vinagre_connection_new_from_string (remaining_args[i], &error);
+	  conn = vinagre_connection_new_from_string (remaining_args[i], &error, TRUE);
 	  if (conn)
 	    servers = g_slist_prepend (servers, conn);
 	  else
 	    errors = g_slist_prepend (errors,
-				      g_strdup_printf ("%s: %s",
+				      g_strdup_printf ("<i>%s</i>: %s",
 						       remaining_args[i],
 						       error ? error : _("Unknown error")));
 
@@ -124,6 +128,7 @@ int main (int argc, char **argv) {
   VinagreWindow     *window;
   VinagreApp        *app;
 
+  setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
@@ -169,8 +174,10 @@ int main (int argc, char **argv) {
   gtk_main ();
 
   g_object_unref (vinagre_bookmarks_get_default ());
-  g_object_unref (vinagre_mdns_get_default ());
   g_object_unref (vinagre_prefs_get_default ());
+#ifdef VINAGRE_ENABLE_AVAHI
+  g_object_unref (vinagre_mdns_get_default ());
+#endif
 
   return 0;
 }
